#include <algorithm> 
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <sstream>
#include <map>
#include <stdexcept>
#include <string>
#include <vector>
#include <iterator>
#include <utility>
#include <time.h>
#include <stdlib.h>
#include <iomanip>


#include "split.h"  
#include <time.h>

using namespace std;

typedef vector<string> Rule;
typedef vector<Rule> Rule_collection; 
typedef map<string, Rule_collection> Grammar; 
long sek = 0;
Grammar gr;

ostream& operator<<(ostream& os, Grammar g) {
	for (Grammar::const_iterator gci=g.begin(); gci != g.end(); gci++) {
		os << setw(12) << left << gci->first << " <- "; 
		Rule_collection rc = gci->second;
		for (Rule_collection::const_iterator itr = rc.begin(); itr != rc.end(); ++itr){
			copy(itr->begin(), itr->end(), ostream_iterator<string>(cout, " "));
			cout << " | ";
		}		
		cout << endl;	
	}	
	return os;
}


// read a grammar from a given input stream
Grammar read_grammar(istream& in)
{
	Grammar ret;
	string line;

	// read the input
	while (getline(in, line)) {

		// `split' the input into words
		vector<string> entry = split(line);

		if (!entry.empty())
			// use the category to store the associated rule
			ret[entry[0]].push_back(
				Rule(entry.begin() + 1, entry.end()));
	}
	return ret;
}

Grammar read_file(char* in_datei) {  
	Grammar ret;
	ifstream in; 
	in.open(in_datei);
	char zeile[256];
	if(!in) {
		cerr << "Fehler beim Oeffnen der Datei " << in_datei << "\n";
		return ret;
	}
	while (in) {
		while(in.getline(zeile,256,'\n')) {
			string str;
			vector<string> entry;
			stringstream ss(zeile);
			ss >> str; entry.push_back(str);
			while (ss >> str) entry.push_back(str);
			ret[entry[0]].push_back(
				Rule(entry.begin() + 1, entry.end()));
		}
	}
	in.close();	
	cout << ret;
	return ret;
}
void gen_aux(const Grammar&, const string&, vector<string>&);

int nrand(int);

vector<string> gen_sentence(const Grammar& g)
{
	vector<string> ret;
	gen_aux(g, "<sentence>", ret);
	return ret;
}

bool bracketed(const string& s)
{
	return s.size() > 1 && s[0] == '<' && s[s.size() - 1] == '>';
}

void
gen_aux(const Grammar& g, const string& word, vector<string>& ret)
{

	if (!bracketed(word)) {
		ret.push_back(word);
	} else {
		// locate the rule that corresponds to `word'
		Grammar::const_iterator it = g.find(word);
		if (it == g.end())
			throw logic_error("empty rule");

		// fetch the set of possible rules
		const Rule_collection& c = it->second;

		// from which we select one at random
		const Rule& r = c[nrand(c.size())];

		// recursively expand the selected rule
		for (Rule::const_iterator i = r.begin(); i != r.end(); ++i)
			gen_aux(g, *i, ret);
	}
}

int main() {
	// read the Gramamar
	gr = read_file("grammatik.txt");
	
	// generate random sentences
	time(&sek); srand( (unsigned)sek);
	
	for (int lp=24; lp>0; lp--) {
		vector<string> sentence = gen_sentence(gr);	 

		// write the first word, if any
		if (!sentence.empty()) cout << "\n" << sentence.front() << " "; 

		// write the rest of the words, each preceded by a space
		copy(++sentence.begin(), sentence.end(), ostream_iterator<string>(cout, " "));
	}
	return 0;
}

// return a random integer in the range `[0,' `n)'
int nrand(int n)
{
	if (n <= 0 || n > RAND_MAX)
		throw domain_error("Argument to nrand is out of range");

	const int bucket_size = RAND_MAX / n;
	int r;

	do r = rand() / bucket_size;
	while (r >= n);

	return r;
}
